// dict.cpp

/* This file contains the lookup and execution functions as well 
 * as loading the dictionary with the commands, the number of 
 * parameters each requires and the help strings.
 */

#include <string.h>  // strcasecmp()
#include <stdio.h> // printf()
#include "dict-util.h" 
#include "ui-tool.h"
#include "help_strings.h"

#define ABS(x) (x < 0 ? -x : x)

extern "C"
{
dict_t dict[] =
{
  // Here's all there is to it.
#define DEF(name, nParams) {#name, (func_t)name, nParams, name##_help},
#include "def_loader.h"
#undef DEF
  // and the dictionary is loaded and all fields are defined.
  0,0
};
  
// We allow case insensitive function name lookups
#define STREQ(a, b) (!strcasecmp(a, b))

// Internal: returns index or -1 if not found
int find_command(const char* cmd)
{
  for(dict_t* p = dict; p->name != 0; p++)
  {
    if(STREQ(p->name, cmd))
      return p - dict;
  }
  return -1;
}

// Internal: executes the command at index (like a byte code)
// with the args from the commandline (not like byte code).
void do_cmd(int cmd_index, char** args)
{
  bool ret = false;
  dict_t* p = dict+cmd_index;
  func_t e = p->func;
  int nparams = ABS(p->argc);
  ret = p->func(args[1], args[2], args[3], args[4]);
/*  
  switch(nparams)
  {
    case 0:
      ret = p->func();
      break;
    case 1:
      ret = p->func(args[1]);
      break;
    case 2:
      ret = p->func(args[1], args[2]);
      break;
    case 3:
      ret = p->func(args[1], args[2], args[3]);
      break;
    case 4:
      ret = p->func(args[1], args[2], args[3], args[3]);
      break;
    default:
      ret = false;
  }
  */
  if(!ret)
    fprintf(stderr, "Unknown error occurred in execution of '%s'\n", p->name);
}
  
void show_command_help(const char* opt_substr)
{
  bool scanstr;
  
  const char* commands_str =
      "ui-tool commands (minus dashes) [substr].\n"
      "   $UIFILE is the name of the qt4 ui file whenever a the ui\n"
      "   file name is required.\n"
      "   Some of the functions accept optional arguments.\n"
      "\n";
  if(opt_substr != 0)
    scanstr = true;
  else
    scanstr = false;
  
  if(!scanstr)  
    printf(commands_str);
  else
    printf("Commands containing substring '%s':\n\n", opt_substr);
  
  for(dict_t* p = dict; p->name != 0; p++)
  {
    if(scanstr)
    {
      if(strcasestr(p->help, opt_substr))
        printf(p->help);
    }
    else
      printf(p->help);
  }
}
 
} // extern "C"
