/****************************************************************** 
  This program does the nuts and bolts of 'mc2 -create ui-files' 
  which will generate a simple C++ file set for a designer or a 
  qtcreator ui file which can be compiled with an mc2 makefile.  

  It will place the C++ sources in the same folder as the ui file.

  This may need to updated later, as we get into creating more 
  complex user interfaces.  QT3 allowed much greater flexibility 
  with subclassing these critters though.  Progess...  Sometimes 
  you wonder.  So it's also possible we may NOT have to update 
  this later.
  
  In any case here's an example of a program we can use as a 
  -create function from mc2.

  Note that the main function is placed in app-main.cpp so as to 
  avoid clobbering a preexisting main.cpp in case there is one.
 ******************************************************************/

#include <string.h> // strncpy()
#include <stdio.h>  // sprintf()
#include <stdlib.h> // exit()
#include <ctype.h>  // toupper(), tolower(), ...

extern void dbg();

// These dat files have '%s' inserted for each parameter
// required.  The parameters are the names of variables 
// that resemble the actual names in terms of upper/lower 
// case but are not the actual variable's values which are
// set up in main().

#include "app_main_cpp.dat"
// s1 = widgetname
// s2 = widgetname
// s3 = WidgetName

// These defines are just for convenience, being the parameters
// in the right order for use by the formats loaded in the dat
// files.
#define app_main_cpp_args widgetname, widgetname, WidgetName

#include "widgetname_cpp.dat"
// s1 = widgetname
// s2 = widgetname
// s3 = widgetname
// s4 = WidgetName, s5 = WidgetName
// s6 = QParentClass
// s7 = WidgetName
// s8 = WidgetName, s9 = WidgetName
// s10 = WidgetName

#define widgetname_cpp_args \
  widgetname, widgetname, widgetname, WidgetName, WidgetName, \
  QParentClass, WidgetName, WidgetName, WidgetName, WidgetName


#include "widgetname_h.dat"
// s1 = widgetname
// s2 = WIDGETNAME
// s3 = WIDGETNAME
// S4 = QParentClass
// s5 = WidgetName
// s6 = WidgetName S7 = QParentClass
// s8 = WidgetName
// s9 = WidgetName
// s10 = WidgetName
// s11 = WIDGETNAME

#define widgetname_h_args \
  widgetname, WIDGETNAME, WIDGETNAME, QParentClass, \
  WidgetName, WidgetName, QParentClass, \
  WidgetName, WidgetName, WidgetName, WIDGETNAME

#define error(s, what) fprintf(stderr, s, what), fprintf(stderr, "\n"), exit(1)

// These are string case converters for generating two new variables 
// from a single WidgetName variable.
static void strupper(char* dest, const char* src)
{
  while(*src)
  {
    *dest++ = toupper(*src++);
  }
  *dest = 0;
}
    
static void strlower(char* dest, const char* src)
{
  while(*src)
  {
    *dest++ = tolower(*src++);
  }
  *dest = 0;
}
    
// returns error code
int usage(int errcode);

// returns 0 on success
int split_path(char* dir, char* file, const char* path);

// returns 0 on success
int get_names(char* QParentClass, char* WidgetName, const char* filename);

// returns 0 on success
int run_uic(const char* uipath, const char* uiname, const char* widgetname);

// returns 0 on success
int run_moc(const char* uipath, const char* widgetname);

// warning if file exists, no overwriting..
void outfile_exists(const char* outfile);


// This name is the interface to these functions so we 
// export it, in essence, as extern "C"
extern "C" 
int ui2cpp_main(int argc, char** argv)
{
  dbg();
  FILE* fp;
  char uipath[256];
  char filename[256];
  char WidgetName[256];
  char widgetname[256];
  char WIDGETNAME[256];
  char QParentClass[256];
  char outfile[256];

  // usage error if no input param
  if(argc != 2)
    return usage(1);
  
  // usage ok if --help
  if(!strcmp(argv[1], "--help"))
    return usage(0);

  // split uipath, uiname
  if(split_path(uipath, filename, argv[1]))
    return usage(1);
  
  // if ui file contains uppercase letters, error
  { 
#define argv1 argv[1];
    char* inname = argv1;
    char buf[256];
    
    // point to basename
    char* p = strrchr(inname, '/');
    if(!p) p = inname;
    else p++;
    
    // convert to lower case name in buf
    int namelen = strlen(inname);
    for(int i = 0; i < namelen; i++)
      buf[i] = tolower(inname[i]);
    buf[namelen] = 0;
        
    // if not equal, show what it should be and exit.
    if(strcmp(buf, inname) != 0)
    {
      fprintf(stderr, 
              "\n"
              "The ui file should be all lower case for this to to work,\n"
              "Recommend: Change ui file name to %s\n"
              "  [example:] mv %s %s\n"
              "Aborting!\n"
              "\n", buf, inname, buf
             );
      exit(1);
    }
    
  }
  
  // read WidgetName and QParentClass from ui file (else error)
  if(get_names(QParentClass, WidgetName, argv[1]))
  {
    usage(1);
    fprintf(stderr, 
            "Note: Can't find class and parent class defs in\n"
            "ui file: '%s'\n", argv[1]
           );
    return 1;
  }
  
  // generate widgetname and WIDGETNAME strings from WidgetName
  strupper(WIDGETNAME, WidgetName);
  strlower(widgetname, WidgetName);
   
  // write ui-main.txt, %s{widgetname}.cpp, and %s{widgetname}.h
  char basename[256];
  strcpy(basename, filename);
  char* p = strrchr(basename, '.');
  if(p)
    *p = 0;
  
  sprintf(outfile, "%s/%s-main.txt", uipath, basename);
  if((fp = fopen(outfile, "r")))
  {
    fclose(fp);
    outfile_exists(outfile);
  }
  else
  {
    fp = fopen(outfile, "w");
    if(!fp)
      error("Can't write %s", outfile);
    fprintf(fp, app_main_cpp, app_main_cpp_args);
    fclose(fp);
  }
  sprintf(outfile, "%s/%s.cpp", uipath, widgetname);
  if((fp = fopen(outfile, "r")))
  {
    fclose(fp);
    outfile_exists(outfile);
  }
  else
  {
    fp = fopen(outfile, "w");
    if(!fp)
      error("Can't write %s", outfile);
    fprintf(fp, widgetname_cpp, widgetname_cpp_args);
    fclose(fp);
  }
  
  sprintf(outfile, "%s/%s.h", uipath, widgetname);
  if((fp = fopen(outfile, "r")))
  {
    fclose(fp);
    outfile_exists(outfile);
  }
  else
  {
    fp = fopen(outfile, "w");
    if(!fp)
      error("Can't write %s", outfile);
    fprintf(fp, widgetname_h, widgetname_h_args);
    fclose(fp);
  }
  
  if(run_uic(uipath, filename, widgetname))
  {
    fprintf(stderr, "Error running uic on '%s'\n", filename);
    return 1;
  }
  
  if(run_moc(uipath, widgetname))
  {
    fprintf(stderr, "Error running moc on '%s.h'\n", widgetname);
    return 1;
  }
  
  if(run_moc(uipath, filename))
  {
    fprintf(stderr, "Error running moc on '%s.h'\n", filename);
    return 1;
  }

  return 0;
}

// returns error code
int usage(int errcode)
{
#include "ui2cpp_usage_str.dat"
  if(errcode)
    fprintf(stderr, "%s", ui2cpp_usage_str);
  else
    fprintf(stdout, "%s", ui2cpp_usage_str);
  return errcode;
}

// returns 0 on success
int split_path(char* dir, char* file, const char* path)
{
  const char* p = strchr(path, '/');
  char* dirptr = dir;
  if(p)
  {
    for(; path < p; path++)
      *dirptr++ = *path;
    *dirptr = 0;
    path++;
  }
  else
    strcpy(dirptr, ".");
  
  strcpy(file, path);
  return 0;
}

// returns 0 on success
int get_names(char* QParentClass, char* WidgetName, const char* filename)
{ 
  // it would be nice to be able to parse the xml file ourselves but 
  // we don't have a standard tool to do that yet, so we'll let QT do it
  // for us by capturing the uic output and filtering for one string 
  // that has both values we need, namely the 'setupUi()' function.
  
  // uic form1.ui | sed '/void setupUi/!d; s|void setupUi(||; s|)||; s|\*||' 
  char buf[4096];
  sprintf(buf, "uic '%s' | sed '/void setupUi/!d; s|void setupUi(||; s|)||; s|\\*||' 2>/dev/null", filename);
  FILE* fp = popen(buf, "r");
  if(!fp)
    return 1;
  *buf = 0;
  fgets(buf, 4096, fp);
  fclose(fp);
  
  ////////////////////
  char* ip = buf;
  char* op;
  
  // skip leading whitespaces if any
  while((*ip <= 32) && (*ip != 0))
    ip++;
  
  // get QParentClass while chars
  op = QParentClass;
  while(*ip > 32)
    *op++ = *ip++;
  *op = 0;
  
  // skip whitespaces
  while((*ip <= 32) && (*ip != 0))
    ip++;
  
  // and not end of input
  if(*ip == 0)
    return 1; // no widget name

  // get WidgetName while chars
  op = WidgetName;
  while(*ip > 32)
    *op++ = *ip++;
  *op = 0;
  
  // error if no WidgetName
  if(*WidgetName == 0)
    return 1;
  
  return 0;
}

// returns 0 on success
int run_uic(const char* uipath, const char* uiname, const char* widgetname)
{
  char cmd[256];
  sprintf(cmd, "uic %s/%s -o %s/ui_%s.h 2>/dev/null", uipath, uiname, uipath, widgetname);
  return system(cmd);
}


// returns 0 on success
int run_moc(const char* uipath, const char* uiname)
{
  char filename[256];
  strcpy(filename, uiname);
  strtok(filename, ".");
  
  char cmd[256];
  sprintf(cmd, "moc %s/%s.h -o %s/moc_%s.cpp >/dev/null 2>&1", uipath, filename, uipath, filename);
  if(system(cmd))
    return 0; // nothing
  // clear the redir
  strchr(cmd, '>')[0] = 0; 
  strcat(cmd, "2>/dev/null");
  return system(cmd);
}

// returns 0 on success
int run_rcc(const char* uipath, const char* filename)
{
  char cmd[256];
  sprintf(cmd, "rcc %s/%s.h -o %s/qrc_%s.qrc >/dev/null 2>&1", uipath, filename, uipath, filename);
  if(system(cmd))
    return 0; // nothing
  return system(cmd);
}

// warning if file exists, no overwriting..
void outfile_exists(const char* outfile)
{
  fprintf(stderr, "NOTICE: %s exists, will not overwrite..\n", outfile);
}
