// ui-tool.cpp
// main.cpp -- skeleton created by new.main
/*
 *   sortSlots \$UIFILE
 *    sorts the slots in a ui file for easier lookups of slots 
 *    in the signal slot editor
 * 
 *  writeCpp \$UIFILE
 *    writes slot templates for the file named like the $UIFILE
 *    but with the '.cpp' file extension.
 *    
 *  writeH \$UIFILE
 *    writes slot templates for the file named like the $UIFILE
 *    but with the '.h' file extension.
 * 
 *  listSlots \$UIFILE
 *    Shows slots found in the \$UIFILE
 * 
 *  listSignals \$UIFILE
 *    Shows signals found in the \$UIFILE
 * 
 *  commands
 *    shows this list and usage notes.
 * 
 *  help
 *    Displays a simple help message in a message box
 */

#include <QMessageBox>
#include <QDesktopWidget>
#include <QApplication>
#include <LQ-qt/SList.h>
#include <stdio.h> // printf()
#include <ctype.h> // char stuff
#include <string.h> // memcmp()
#include "dict-util.h" // help lookups

#define NO_PARSER_WARN 1
#include <LQ-qt/Parser.h>

using namespace Parser;

// to avoid the name mangling
extern "C"
{

void centerWindow(QWidget* window)
{
  // Unreliable if window isn't shown yet
  if(!window->isVisible()) return;
  
  int dtx,dty,dtw,dth;
  int winx,winy,winw,winh;
  int x, y;
  
  QDesktopWidget* dt = qApp->desktop();
  dt->availableGeometry().getRect(&dtx,&dty,&dtw,&dth);
  window->geometry().getRect(&winx,&winy,&winw,&winh);
  
  x = dtx + dtw/2 - winw/2; // x1 should not always be 0
  y = dty + dth/2 - winh/2; // y1 ""
  
  window->setGeometry(x, y, winw, winh);
  qApp->processEvents();
}


void doMsg(QMessageBox* b)
{
  b->setWindowFlags(Qt::WindowStaysOnTopHint);
  b->show();
  centerWindow(b);
  b->exec();
}

// requires instance of QApplication in main()
void errorMsg(const char* msg)
{  
  QMessageBox* b = new QMessageBox(QMessageBox::Critical, "Error", msg,
                                   QMessageBox::Ok, NULL);
  doMsg(b);
}

// requires instance of QApplication in main()
void warnMsg(const char* msg)
{
  QMessageBox* b = new QMessageBox(QMessageBox::Warning, "Error", msg,
                                   QMessageBox::Ok, NULL);
  doMsg(b);
}

// requires instance of QApplication in main()
void infoMsg(const char* msg)
{
  QMessageBox* b = new QMessageBox(QMessageBox::Information, "Notice", msg,
                                   QMessageBox::Ok);
  doMsg(b);
}


int usage(int errcode)
{
#include "uitool_usage_str.dat"
  if(errcode)
    errorMsg(uitool_usage_str);
  else
    infoMsg(uitool_usage_str);
  return errcode;
}

void TODO(const char* funcname)
{
  char buf[256];
  sprintf(buf, 
          "%s is not yet implemented.\n"
          "For now, you may have to use a text\n"
          "editor and do this operation manually.\n"
          , funcname);
  errorMsg(buf);
  exit(1);
}

} // extern "C"

